package com.gigachatwoot.connectors.chatwoot.controllers;

import com.gigachatwoot.connectors.chatwoot.dto.CSMLFlowInfoDTO;
import com.gigachatwoot.connectors.chatwoot.services.BotDefinitionService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

@RestController
@RequestMapping("/csml/flows")
@RequiredArgsConstructor
@Slf4j
public class CSMLFlowController {

    private final BotDefinitionService botDefinitionService;

    @PostMapping
    public ResponseEntity<String> updateFlows(@RequestBody CSMLFlowInfoDTO updateRequest) {
        try {
            botDefinitionService.updateBotFlows(updateRequest);
            return ResponseEntity.ok("CSML flows updated successfully.");
        } catch (IllegalArgumentException e) {
            log.error("Validation error: {}", e.getMessage());
            return ResponseEntity.status(HttpStatus.BAD_REQUEST).body(e.getMessage());
        } catch (IllegalStateException e) {
            log.error("CSML validation error: {}", e.getMessage());
            return ResponseEntity.status(HttpStatus.BAD_REQUEST).body("CSML validation failed: " + e.getMessage());
        } catch (Exception e) {
            log.error("Error updating CSML flows: {}", e.getMessage(), e);
            return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR)
                .body("Failed to update CSML flows.");
        }
    }

    @GetMapping
    public ResponseEntity<CSMLFlowInfoDTO> getCurrentFlowInfo() {
        try {
            CSMLFlowInfoDTO flowInfo = botDefinitionService.getFlowInfo();
            return ResponseEntity.ok(flowInfo);
        } catch (Exception e) {
            log.error("Error retrieving CSML flow info: {}", e.getMessage(), e);
            return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR)
                .body(null);
        }
    }
}

