package com.gigachatwoot.connectors.chatwoot.handlers;

import an.awesome.pipelinr.Command;
import an.awesome.pipelinr.Voidy;
import com.gigachatwoot.connectors.chatwoot.client.CSMLClient;
import com.gigachatwoot.connectors.chatwoot.client.ChatwootClient;
import com.gigachatwoot.connectors.chatwoot.commands.ProcessChatwootMessageCommand;
import com.gigachatwoot.connectors.chatwoot.dto.ChatwootWebhookPayloadDTO;
import com.gigachatwoot.connectors.chatwoot.models.ChatMessage;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.PropertySource;
import org.springframework.stereotype.Component;

import java.util.List;

@Slf4j
@RequiredArgsConstructor
@Component
@PropertySource("classpath:application.yml")
public class ProcessChatwootMessageHandler implements Command.Handler<ProcessChatwootMessageCommand, Voidy> {
    @Value("${chatwoot.api.access.token.user}")
    private String apiAccessTokenUser;

    @Value("${chatwoot.api.base.url}")
    private String chatwootUrl;

    @Value("${csml.api.base.url}")
    private String csmlUrl;

    private final ChatwootClient chatwootClient;
    private final CSMLClient csmlClient;

    @Override
    public Voidy handle(ProcessChatwootMessageCommand command) {
        ChatwootWebhookPayloadDTO payload = command.getPayload();

        // Check if assignee is null and client requests an operator
        if (clientRequestsOperator(payload)) {
            Long agentId = chatwootClient.getRandomAvailableAgentId(
                payload.getAccountId(),
                apiAccessTokenUser,
                chatwootUrl);
            if (agentId != null) {
                chatwootClient.assignConversationToAgent(
                    payload.getAccountId(),
                    payload.getConversationId(),
                    agentId,
                    apiAccessTokenUser,
                    chatwootUrl);
                log.info("handle: Assigned agent with ID {} to conversation {}", agentId, payload.getConversationId());
                return new Voidy();
            } else {
                log.warn("handle: No available agents to assign.");
            }
        }

        // Get message history
        List<ChatMessage> messages = chatwootClient
            .getMessageHistory(
                payload.getAccountId(),
                payload.getConversationId(),
                apiAccessTokenUser,
                chatwootUrl);
        if (messages.isEmpty()) {
            log.warn("handle: No messages found in conversation history.");
            return new Voidy();
        }

        // Sort and filter messages
        List<ChatMessage> sortedMessages = chatwootClient.sortAndFilterMessages(messages);
        if (sortedMessages.isEmpty()) {
            log.warn("handle: No valid messages found after filtering.");
            return new Voidy();
        }

        // Отправка истории сообщений в CSML

       csmlClient.sendMessageHistoryToCSML(
            payload.getAccountId(),
            payload.getConversationId(),
            sortedMessages,
            csmlUrl
       );

        return new Voidy();
    }

    private boolean clientRequestsOperator(ChatwootWebhookPayloadDTO payload) {
        // Check if the message content contains the keyword "/operator"
        String content = payload.getContent().toLowerCase();
        return content.contains("/operator");
    }
}
