package com.gigachatwoot.connectors.chatwoot.services;

import com.gigachatwoot.connectors.chatwoot.client.ChatwootClient;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.PropertySource;
import org.springframework.stereotype.Service;

import java.util.Map;

@Slf4j
@Service
@RequiredArgsConstructor
@PropertySource("classpath:application.yml")
public class CSMLResponseService {
    private final ChatwootClient chatwootClient;

    @Value("${chatwoot.api.access.token.agent.bot}")
    private String apiAccessTokenAgentBot;

    @Value("${chatwoot.api.access.token.user}")
    private String apiAccessTokenUser;

    @Value("${chatwoot.api.base.url}")
    private String apiBaseUrl;

    public void handleCSMLResponse(Map<String, Object> requestBody) {
        Long accountId = null;
        Long conversationId = null;
        System.out.println(requestBody.get("needs_operator"));
        Boolean needsOperator = (Boolean) requestBody.get("needs_operator");

        try {
            accountId = Long.parseLong((String) requestBody.get("account_id"));
            conversationId = Long.parseLong((String) requestBody.get("conversation_id"));
        } catch (Exception e) {
            log.error("handleCSMLResponse: " +
                "Failed to parse account_id or conversation_id to Long or needs_operator to Boolean", e);
        }

        String message = (String) requestBody.get("message");

        log.info("Received data - accountId: {}, conversationId: {}, needsOperator: {}, message: {}",
            accountId, conversationId, needsOperator, message);

        chatwootClient.sendMessageToChatwoot(accountId, conversationId, message, apiAccessTokenAgentBot, apiBaseUrl);

        if(Boolean.TRUE.equals(needsOperator)) {
            Long agentId = chatwootClient.getRandomAvailableAgentId(
                accountId,
                apiAccessTokenUser,
                apiBaseUrl);
            if (agentId != null) {
                chatwootClient.assignConversationToAgent(
                    accountId,
                    conversationId,
                    agentId,
                    apiAccessTokenUser,
                    apiBaseUrl);
                log.info("handleCSMLResponse: Assigned agent with ID {} to conversation {}", agentId, conversationId);
            } else {
                log.warn("handleCSMLResponse: No available agents to assign.");
            }
        }
    }
}
