package com.gigachatwoot.connectors.chatwoot.services;

import an.awesome.pipelinr.Pipeline;
import com.gigachatwoot.connectors.chatwoot.commands.ProcessChatwootMessageCommand;
import com.gigachatwoot.connectors.chatwoot.dto.ChatwootWebhookPayloadDTO;
import com.gigachatwoot.connectors.chatwoot.models.ChatwootWebhookPayload;
import com.gigachatwoot.connectors.chatwoot.mapper.ChatwootWebhookPayloadMapper;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.PropertySource;
import org.springframework.stereotype.Service;

@Slf4j
@Service
@RequiredArgsConstructor
public class ChatwootWebhookService {
    private final static String INCOMING_MESSAGE_KEY = "incoming";

    private final Pipeline pipeline;

    public void handleWebhook(ChatwootWebhookPayload payload) {
        // Log received payload for debugging
        log.info("handleWebhook: Received webhook from Chatwoot: {}", payload);

        // Check for conversation
        if (payload.getConversation() == null) {
            log.warn("handleWebhook: No conversation found in payload.");
            return;
        }

        // Check for meta presence inside conversation
        if (payload.getConversation().getMeta() == null) {
            log.warn("handleWebhook: No meta data of conversation found in payload.");
            return;
        }

        if(payload.getConversation().getMeta().getAssignee() != null) {
            log.warn("handleWebhook: Received message is tied to the operator and will not be sent to GigaChat");
            return;
        }

        // Process only incoming messages
        if (!INCOMING_MESSAGE_KEY.equals(payload.getMessageType())) {
            log.warn("handleWebhook: Message type is not incoming. Ignoring message.");
            return;
        }

        processWebhook(ChatwootWebhookPayloadMapper.toDTO(payload));
    }

    private void processWebhook(ChatwootWebhookPayloadDTO payload) {
        // Create and send the command through the pipeline
        ProcessChatwootMessageCommand command = new ProcessChatwootMessageCommand(payload);
        pipeline.send(command);
    }
}
