package com.gigachatwoot.connectors.gigachat.client;

import com.fasterxml.jackson.databind.JsonNode;
import com.gigachatwoot.connectors.chatwoot.models.ChatMessage;
import com.gigachatwoot.connectors.gigachat.mapper.UnirestMapper;
import com.gigachatwoot.connectors.gigachat.dto.GigaChatAuthDTO;
import com.gigachatwoot.connectors.gigachat.dto.GigaChatRequestDTO;
import com.gigachatwoot.connectors.gigachat.models.GigaChatAuthRequestModel;
import com.gigachatwoot.connectors.gigachat.models.GigaChatMessageRequestModel;
import jakarta.validation.constraints.NotNull;
import kong.unirest.HttpResponse;
import kong.unirest.Unirest;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.PropertySource;
import org.springframework.stereotype.Component;

import java.util.List;

import static org.springframework.http.MediaType.APPLICATION_JSON_VALUE;
import static org.springframework.http.MediaType.APPLICATION_FORM_URLENCODED_VALUE;

@Slf4j
@Component
@RequiredArgsConstructor
@PropertySource("classpath:application.yml")
public class GigaChatConnectorClient {
    @Value("${gigachat.chat_url}")
    @Getter
    @NotNull(message = "Chat URL is required.")
    private String chatUrl;

    @Value("${gigachat.oauth_url}")
    @Getter
    @NotNull(message = "OAUTH URL is required.")
    private String oauthUrl;

    private final GigaChatAuthRequestModel authRequest;

    private final GigaChatAuthDTO authDTO;

    private final GigaChatMessageRequestModel msgRequest;

    private final UnirestMapper objectMapper;

    public String getAccessToken() {
        HttpResponse<String> response = Unirest.post(getOauthUrl())
            .header("Content-Type", APPLICATION_FORM_URLENCODED_VALUE)
            .header("Accept", APPLICATION_JSON_VALUE)
            .header("RqUID", authRequest.getRquid())
            .header("Authorization", authRequest.getAuthorization())
            .field("scope", authDTO.getScope())
            .asString();

        return parseAccessToken(
            objectMapper.readValue(response.getBody(), JsonNode.class)
        );
    }

    public String getGigaChatAnswer(List<ChatMessage> messageHistory) {
        String accessTokenResponse = getAccessToken();

        HttpResponse<String> response = Unirest.post(getChatUrl())
            .header("Content-Type", APPLICATION_JSON_VALUE)
            .header("Accept", APPLICATION_JSON_VALUE)
            .header("Authorization", "Bearer " + accessTokenResponse)
            .body(messageToString(msgRequest.getSystemPrompt(), messageHistory))
            .asString();

        return parseGigaChatContent(
            objectMapper.readValue(response.getBody(), JsonNode.class)
        );
    }

    private String messageToString(@NotNull String systemPrompt, @NotNull List<ChatMessage> messageHistory) {
        GigaChatRequestDTO output = new GigaChatRequestDTO(systemPrompt, messageHistory);

        return objectMapper.writeValue(output);
    }

    public String parseAccessToken(@NotNull JsonNode accessToken) {
        return accessToken.get("access_token").textValue();
    }

    public String parseGigaChatContent(@NotNull JsonNode response) {
        return response.findValue("content").textValue();
    }
}
